import docker
import json
import os
import tarfile

import constants

def get_client():
    client = docker.from_env()
    return client


def list_testcases(version):
    cwd = os.path.dirname(os.path.abspath(__file__))
    with open(os.path.join(cwd, constants.TESTCASES_FILE)) as f:
        testcases = json.load(f)
    return testcases
    

def run_testcases(container_name, dovetail_home, version, testcases):
    client = get_client()
    envs = ["DOVETAIL_HOME={}".format(dovetail_home)]
    volumes = {dovetail_home: {'bind': dovetail_home, 'mode': 'rw'},
               '/var/run/docker.sock': {'bind': '/var/run/docker.sock', 'mode': 'ro'}}
    dovetail_container = client.containers.run(image="opnfv/dovetail:{}".format(version),
                                               command="/bin/bash",
                                               privileged=True,
                                               detach=True,
                                               tty=True,
                                               environment=envs,
                                               volumes=volumes,
                                               name=container_name)
    tc_str = ""
    for tc in testcases:
        tc_str = tc_str + " --testcase " + tc
    cmd = '/bin/bash -c "dovetail run {} --offline -d -r"'.format(tc_str)
    exit_code, output = dovetail_container.exec_run(cmd=cmd,
                                                    privileged=True,
                                                    stream=True)
    return exit_code


def clean_container(container_name):
    client = get_client()
    try:
        container = client.containers.get(container_name)
        container.remove(force=True)
    except docker.errors.NotFound:
        return True
    except docker.errors.APIError:
        return False
    return True
