#!flask/bin/python

import os
import time
import uuid

from flask import Flask, jsonify, abort, make_response, request

from app import app
import constants
import docker_utils
import utils


#@app.errorhandler(400)
#def error_request(error):
#    return make_response(jsonify({'error': 'Data in request could not be recognised.'}), 400)


@app.route('/api/v1/scenario/nfvi/testsuites', methods=['GET'])
def get_all_testsuites():
    return jsonify({'testsuites': constants.TESTSUITES}), 200


@app.route('/api/v1/scenario/nfvi/testcases', methods=['GET'])
def get_testcases():
    testcases = docker_utils.list_testcases(version=constants.VERSION)
    return jsonify({'testcases': testcases}), 200


@app.route('/api/v1/scenario/nfvi/execution', methods=['POST'])
def run_testcases():
    requestId = request.args.get('requestId')
    if not requestId:
        requestId = uuid.uuid1()
    container_name = 'dovetail-{}'.format(requestId)
    dovetail_home = '/tmp/dovetail-{}'.format(requestId)

    env_set = constants.MIN_ENV_SET
    for item in env_set:
        if not item in request.json['conf']['envs']:
            abort(400)
    env_file = os.path.join(dovetail_home, 'pre_config', 'env_config.sh')
    utils.write_env_file(request.json['conf']['envs'], env_file)
    pod_file = os.path.join(dovetail_home, 'pre_config', 'pod.yaml')
    utils.write_pod_file(request.json['conf']['pods'], pod_file)
    tempest_conf_file = os.path.join(dovetail_home, 'pre_config', 'tempest_conf.yaml')
    utils.write_tempest_conf_file(request.json['conf']['tempest_conf'], tempest_conf_file)

    utils.copy_vm_images(request.json['conf']['vm_images'],
                         os.path.join(dovetail_home, 'images'))

    testcases = [tc['testCaseName'] for tc in request.json['testcases']]
    docker_version = constants.VERSION['docker']
    docker_utils.run_testcases(container_name=container_name,
                               dovetail_home=dovetail_home,
                               version=docker_version,
                               testcases=testcases)

    result = utils.get_execution_status(os.path.join(dovetail_home, 'results'),
                                        testcases, requestId)
    return jsonify({'result': result}), 200


@app.route('/api/v1/scenario/nfvi/execution/status/<exec_id>', methods=['POST'])
def get_testcases_status(exec_id):
    if not 'testcases' in request.json:
        abort(400)
    testcases = request.json['testcases']
    dovetail_home = '/tmp/dovetail-{}'.format(exec_id)
    container_name = 'dovetail-{}'.format(exec_id)
    result = utils.get_execution_status(os.path.join(dovetail_home, 'results'),
                                        testcases, exec_id)
    return jsonify({'result': result}), 200


@app.route('/api/v1/scenario/nfvi/clean/<exec_id>', methods=['GET'])
def clean_resources(exec_id):
    container_name = 'dovetail-{}'.format(exec_id)
    dovetail_home = '/tmp/dovetail-{}'.format(exec_id)

    msg = "OK"
    code = 200
    res = docker_utils.clean_container(container_name)
    if not res:
        msg = "Failed to clean container {}.".format(container_name)
        code = 500
    utils.clean_workspace(dovetail_home)
    return jsonify({'result': msg}), code


@app.route('/api/v1/scenario/nfvi/delete/<exec_id>', methods=['GET'])
def delete_all(exec_id):
    dovetail_home = '/tmp/dovetail-{}'.format(exec_id)

    utils.clean_home(dovetail_home)
    return jsonify({'result': "ok"}), 200


@app.route('/api/v1/package/<exec_id>', methods=['GET'])
def package_vtp_results(exec_id):
    vtp_home = '/tmp/vtp-{}'.format(exec_id)
    res, msg = utils.package_vtp_results(vtp_home, exec_id)
    code = 200 if res else 500
    return jsonify({'result': msg}), code
